# -*- coding: utf-8 -*-

# Define self package variable
__version__ = "0.1.a2"
__all__ = ["NanoCount", "Read"]
__description__ = 'EM based transcript abundance from nanopore reads mapped to a transcriptome with minimap2'
